@@ -338,11 +338,12 @@ pango_read_line (FILE *stream, GString *str)
 	      if (!comment)
 		quoted = TRUE;
 	      break;
+	    case '\r':
 	    case '\n':
 	      {
 		int next_c = getc_unlocked (stream);
 
-		if (!(c == EOF ||
+		if (!(next_c == EOF ||
 		      (c == '\r' && next_c == '\n') ||
 		      (c == '\n' && next_c == '\r')))
 		  ungetc (next_c, stream);
@@ -792,6 +793,9 @@ pango_lookup_aliases (const char   *fontname,
   *n_families = 0;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+
 /**
  * pango_find_base_dir:
  * @text:   the text to process
@@ -830,6 +834,8 @@ pango_find_base_dir (const gchar *text,
   return dir;
 }
 
+#pragma GCC diagnostic pop
+
 /**
  * pango_is_zero_width:
  * @ch: a Unicode character
